-------------------------------
-- Created by Tinuviel, cpc197c@gmail.com
--
-- Created under the Creative Commons CC BY-NC license. This license allows for 
-- copying, redistributing, modifying, and building upon as long as this license
-- follows all derivative works and attributes the original author(s)
--     See https://creativecommons.org/licenses/by-nc/4.0/ for more information.
-------------------------------

Eyefinity = {}
Eyefinity.__index = Eyefinity
Eyefinity.AddonID = "EyefinitySupport"

Eyefinity.ToggleRegistration = function()
    if (WF_GetOrDefault( true, WF_SavedVars.MultiScreen["Enabled"])) then
        Eyefinity.Register()
    else
        Eyefinity.UnRegister()
    end
end

Eyefinity.OnIsManualOverride = function(value)
    if (EF_Settings.SetSetting("IsManualOverride", value)) then
        Eyefinity.ToggleRegistration()
    end
end

Eyefinity.OnSetBezelWidth = function(value)
    if (EF_Settings.SetSetting("BezelSize", value) and EF_Settings.GetSetting("IsManualOverride")) then
        Eyefinity.ToggleRegistration()
    end
end

Eyefinity.Register = function()
    EF_Utilities.WriteDebug("->Register")
    EF_Settings.EnsureSavedVariablesLoaded()

    local correctedWitdh = EF_Settings.Default.MonitorWidth
    local bezelCorrection = EF_Settings.Default.BezelCorrection
    if (EF_Settings.GetSetting("IsManualOverride")) then
        bezelCorrection = EF_Settings.GetSetting("BezelSize") / 2
    end

    Eyefinity.AddBezelCorrection(correctedWitdh, bezelCorrection)
    Eyefinity.FixOverflowingUIElements()
    Eyefinity.RegisterConversationElements(correctedWitdh, bezelCorrection)
    Eyefinity.HookTooltipCreation()
end

Eyefinity.UnRegister = function()
    if (Eyefinity.OriginalConversationElements == nil) then
        return
    end

    -- wykkyd handles reset of root UI anchors, just need to unhook from convo elements
    Eyefinity.RevertOverflowingUIElements()
    Eyefinity.UnRegisterConversationElements()
    Eyefinity.UnhookTooltipCreation()
end

Eyefinity.AddBezelCorrection = function(monitorWidth, bezelCorrection)
    if _G["WF_GuiRoot"] == nil then
        zo_callLater(function() Eyefinity.AddBezelCorrection(monitorWidth, bezelCorrection) end, 500)
        return
    end

    GuiRoot:SetWidth( monitorWidth )
    GuiRoot:SetAnchor( LEFT, WF_GuiRoot, LEFT, (monitorWidth) + bezelCorrection, offsety)
end

Eyefinity.FixOverflowingUIElements = function()
    Eyefinity.FixLeftElement()

    if (Eyefinity.OriginalUIElementSettings.RightPanelBackgroundLeftWidth == nil) then
        Eyefinity.OriginalUIElementSettings.RightPanelBackgroundLeftWidth = ZO_SharedRightPanelBackgroundLeft:GetWidth()
        Eyefinity.OriginalUIElementSettings.StatsBackgroundBGWidth = ZO_SharedStatsBackgroundBG:GetWidth()
        Eyefinity.OriginalUIElementSettings.MapBackgroundWidth = ZO_WorldMapInfoFootPrintBackgroundBG:GetWidth()
    end

    ZO_SharedRightPanelBackgroundLeft:SetWidth(Eyefinity.OriginalUIElementSettings.RightPanelBackgroundLeftWidth - 385)
    ZO_SharedStatsBackgroundBG:SetWidth(Eyefinity.OriginalUIElementSettings.StatsBackgroundBGWidth - 305)
    ZO_WorldMapInfoFootPrintBackgroundBG:SetWidth(Eyefinity.OriginalUIElementSettings.MapBackgroundWidth - 600)
end

Eyefinity.RevertOverflowingUIElements = function()
    if (Eyefinity.OriginalUIElementSettings == nil) then
        return
    end
    Eyefinity.RevertLeftElement()

    ZO_SharedRightPanelBackgroundLeft:SetWidth(Eyefinity.OriginalUIElementSettings.RightPanelBackgroundLeftWidth)
    ZO_SharedStatsBackgroundBG:SetWidth(Eyefinity.OriginalUIElementSettings.StatsBackgroundBGWidth)
    ZO_WorldMapInfoFootPrintBackgroundBG:SetWidth(Eyefinity.OriginalUIElementSettings.MapBackgroundWidth)
end

Eyefinity.FixLeftElement = function()
    local ___, opoint, orelativeto, orelativepoint, ooffsetx, ooffsety = ZO_SharedThinLeftPanelBackgroundRight:GetAnchor(9)

    if (Eyefinity.OriginalUIElementSettings == nil) then
        Eyefinity.OriginalUIElementSettings =
        {
            LeftBackgroundRightAnchor = {
                point = opoint, relativeto = orelativeto, relativepoint = orelativepoint, offsetx = ooffsetx, offsety = ooffsety
            },
            LeftBackgroundRightWidth = ZO_SharedThinLeftPanelBackgroundRight:GetWidth(),
            LeftBackgroundLeftWidth = ZO_SharedThinLeftPanelBackgroundLeft:GetWidth(),
            LeftBackgroundRightHeight = ZO_SharedThinLeftPanelBackgroundRight:GetHeight(),
            LeftBackgroundLeftHeight = ZO_SharedThinLeftPanelBackgroundLeft:GetHeight()
        }
    end

    ZO_SharedThinLeftPanelBackgroundRight:ClearAnchors()
    ZO_SharedThinLeftPanelBackgroundRight:SetAnchor(TOPRIGHT, ZO_SharedThinLeftPanelBackground, TOPRIGHT,165,0)
    ZO_SharedThinLeftPanelBackgroundLeft:SetHeight(ZO_SharedThinLeftPanelBackground:GetHeight() - 125)
    ZO_SharedThinLeftPanelBackgroundRight:SetHeight(ZO_SharedThinLeftPanelBackground:GetHeight() - 125)
    ZO_SharedThinLeftPanelBackgroundLeft:SetWidth(ZO_Character:GetWidth() + 20)
    ZO_SharedThinLeftPanelBackgroundRight:SetWidth(ZO_Character:GetWidth() + 20)
end

Eyefinity.RevertLeftElement = function()
    local anch = Eyefinity.OriginalUIElementSettings.LeftBackgroundRightAnchor
    ZO_SharedThinLeftPanelBackgroundRight:ClearAnchors()
    ZO_SharedThinLeftPanelBackgroundRight:SetAnchor(anch.point, anch.relativeto, anch.relativepoint, anch.offsetx, anch.offsety)
    ZO_SharedThinLeftPanelBackgroundLeft:SetHeight(Eyefinity.OriginalUIElementSettings.LeftBackgroundLeftHeight)
    ZO_SharedThinLeftPanelBackgroundRight:SetHeight(Eyefinity.OriginalUIElementSettings.LeftBackgroundRightHeight)
    ZO_SharedThinLeftPanelBackgroundLeft:SetWidth(Eyefinity.OriginalUIElementSettings.LeftBackgroundLeftWidth)
    ZO_SharedThinLeftPanelBackgroundRight:SetWidth(Eyefinity.OriginalUIElementSettings.LeftBackgroundRightWidth)
end

-- As of now, just dialog choices
Eyefinity.RegisterConversationElements = function(correctedWidth, bezelCorrection)
    local ___, uiHeight = GuiRoot:GetDimensions()
    -- Override the divider anchors and width.  This is the horizontal placement
    local ___, dpoint, drelativeto, drelativepoint, doffsetx, doffsety = ZO_InteractWindowDivider:GetAnchor(8)
    local ___, tpoint, trelativeto, trelativepoint, toffsetx, toffsety = ZO_InteractWindowTopBG:GetAnchor(3)
    if (Eyefinity.OriginalConversationElements == nil) then
        Eyefinity.OriginalConversationElements =
        {
            OriginalDividerAnchor = {
                point = dpoint, relativeto = drelativeto, relativepoint = drelativepoint, offsetx = doffsetx, offsety = doffsety
            },
            OriginalTopBGAnchor = {
                point = tpoint, relativeto = trelativeto, relativepoint = trelativepoint, offsetx = toffsetx, offsety = toffsety
            },
            OriginalPopulateFunction = ZO_InteractionManager.PopulateChatterOptions,
            OriginalInteractionWidth = ZO_InteractWindowDivider:GetWidth()
        }
    end

    ZO_InteractWindowDivider:ClearAnchors()

    ZO_InteractWindowDivider:SetAnchor(RIGHT, GuiRoot, TOPRIGHT,  0, uiHeight * .5)
    local interactionElementWidth = correctedWidth / 2
    ZO_InteractWindowDivider:SetWidth(interactionElementWidth)

    -- Set the top background anchor
    ZO_InteractWindowTopBG:SetAnchor(BOTTOMRIGHT, GuiRoot, RIGHT, 0, 0)

    -- Fix dialog choices
    --ZO_InteractWindowPlayerAreaOptions:SetAnchor(3, ZO_InteractWindowPlayerArea, 3, -correctedWidth, 0)
    --ZO_InteractWindowPlayerAreaOptions:SetWidth(interactionElementWidth - correctedWidth * 2)

    -- Hook into PopulateChatterOptions to set to bottom background width
    local originalPopulate = ZO_InteractionManager.PopulateChatterOptions
    ZO_InteractionManager.PopulateChatterOptions = function(optionCount, backToTOCOption)
        originalPopulate(optionCount, backToTOCOption)
        ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0)
    end

    -- More hooks for bottom background width
    EVENT_MANAGER:RegisterForEvent("EyefinitySupport", EVENT_QUEST_OFFERED, function()
        ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0)
    end)
    EVENT_MANAGER:RegisterForEvent("EyefinitySupport", EVENT_QUEST_COMPLETE_DIALOG, function()
        ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0)
    end)
end

Eyefinity.UnRegisterConversationElements = function()
    local divider = Eyefinity.OriginalConversationElements.OriginalDividerAnchor
    local topBg = Eyefinity.OriginalConversationElements.OriginalTopBGAnchor
    local populateFunc = Eyefinity.OriginalConversationElements.OriginalPopulateFunction

    EF_Utilities.WriteDebug(divider)

    --ZO_InteractWindowDivider:ClearAnchors()
    --ZO_InteractWindowDivider:SetAnchor(divider.point, divider.relativeto, divider.relativepoint, divider.offsetx, divider.offsety)
    ZO_InteractWindowDivider:SetWidth(Eyefinity.OriginalConversationElements.OriginalInteractionWidth)
    --ZO_InteractWindowTopBG:SetAnchor(topBg.point, topBg.relativeto, topBg.relativepoint, topBg.offsetx, topBg.offsety)
    ZO_InteractionManager.PopulateChatterOptions = populateFunc

    EVENT_MANAGER:UnregisterForEvent("EyefinitySupport", EVENT_QUEST_OFFERED)
    EVENT_MANAGER:UnregisterForEvent("EyefinitySupport", EVENT_QUEST_COMPLETE_DIALOG)
end

Eyefinity.HookTooltipCreation = function()
    Eyefinity.OriginalTooltipHook = ZO_Tooltips_SetupDynamicTooltipAnchors

    ZO_Tooltips_SetupDynamicTooltipAnchors = function(tooltipUsed, buttonPart, comparativeTooltip1, comparativeTooltip2)
        Eyefinity.OriginalTooltipHook(tooltipUsed, buttonPart, comparativeTooltip1, comparativeTooltip2)
        local anchorPos = buttonPart:GetType()

        --Switch right to left anchor
        if (anchorPos == 7) then
            tooltipUsed:ClearAnchors()
            tooltipUsed:SetAnchor(LEFT, buttonPart, RIGHT, 10, 0)
        end
    end
end

Eyefinity.UnhookTooltipCreation = function()
    ZO_Tooltips_SetupDynamicTooltipAnchors = Eyefinity.OriginalTooltipHook
end