-------------------------------
-- Created by Tinuviel, cpc197c@gmail.com
--
-- Created under the Creative Commons CC BY-NC license. This license allows for 
-- copying, redistributing, modifying, and building upon as long as this license
-- follows all derivative works and attributes the original author(s)
--     See https://creativecommons.org/licenses/by-nc/4.0/ for more information.
-------------------------------

-- Utility Functions!
EF_Utilities = {}
EF_Utilities.IsDebug = false

EF_Utilities.WriteDebug = function(text)
    if (EF_Utilities.IsDebug) then
        zo_callLater(function() d(text) end, 4000)
    end
end

EF_Utilities.AspectRatios =
{
    1.33,   --4:3
    1.5,    --3:2 (old TV's)
    1.6,    --16:10
    1.78    --16:9
}

EF_Utilities.Round = function(number, decimalPlaces)
    local multiple = 10^(decimalPlaces or 0)
    return math.floor(number * multiple + 0.5) / multiple
end

EF_Utilities.GetMonitorWidth= function()
    if (EF_Utilities.OriginalWidth == nil) then
        EF_Utilities.OriginalWidth, EF_Utilities.OriginalHeight = GuiRoot:GetDimensions()
    end

    local uiWidth = EF_Utilities.Round(EF_Utilities.OriginalWidth,0)
    local monitorHeight = EF_Utilities.Round(EF_Utilities.OriginalHeight,0)
    local monitorWidth = uiWidth / 3
    local aspectRatio = EF_Utilities.Round(monitorWidth / monitorHeight, 2)
    local correctedRatio = 0
    EF_Utilities.WriteDebug("UI Width: " .. tostring(uiWidth))
    EF_Utilities.WriteDebug("Monitor Width: " .. tostring(monitorWidth))
    EF_Utilities.WriteDebug("Monitor Height: " .. tostring(monitorHeight))
    EF_Utilities.WriteDebug("Aspect Ratio: " .. tostring(aspectRatio))

    local hasGoodRatio = false;
    for i, val in ipairs(EF_Utilities.AspectRatios) do
        if val == aspectRatio then
            hasGoodRatio = true
            break
        end
    end

    if (not hasGoodRatio) then
        -- If not one of those ratios, this is a custom ratio. Get the ratio closest, then normalize
        for i, val in ipairs(EF_Utilities.AspectRatios) do
            if aspectRatio > val and val > correctedRatio then
                correctedRatio = val
            end
        end
    else
        correctedRatio = aspectRatio
    end

    -- Fix the repeating ratios
    if (correctedRatio == 1.78) then
        correctedRatio = 1.7777777777777777777
    end
    if (correctedRatio == 1.33) then
        correctedRatio = 1.3333333333333333333
    end
    EF_Utilities.WriteDebug("Corrected Ratio: " .. tostring(aspectRatio))

    local correctedMonitorWidth = EF_Utilities.Round(correctedRatio * monitorHeight, 0)
    local bezelWidth = EF_Utilities.Round(uiWidth - (correctedMonitorWidth * 3))
    local bezelCorrection = 0
    if (bezelWidth > 0) then
        bezelCorrection = bezelWidth / 2
    end
    EF_Utilities.WriteDebug("Corrected Monitor Width: " .. tostring(correctedMonitorWidth))
    EF_Utilities.WriteDebug("Bezel Width: " .. tostring(bezelWidth))
    EF_Utilities.WriteDebug("Bezel Correction: " .. tostring(bezelCorrection))

    return correctedMonitorWidth, bezelCorrection, bezelWidth
end